const path = require('path');
const fs = require('fs');
const dotenv = require('dotenv');
const backendPath = path.dirname(process.argv[1]);

class EnvBuilder {
  static DEFAULT_VALUES = {
    LOG_FOLDER: path.join(backendPath, 'logs'),
  }

  static async fileExists(filePath) {
    return new Promise((resolve) => {
      fs.access(filePath, fs.constants.F_OK, (err) => {
        if(err) return resolve(false);
        resolve(true);
      });
    })
  }

  static async readFile(filePath) {
    return new Promise((resolve, reject) => {
      fs.readFile(filePath, 'utf8', (err, data) => {
        if(err) return reject(new Error(`Erro ao ler o arquivo: ${filePath}`));
        resolve(data);
      });
    })
  }

  static async writeFile(filePath, content) {
    return new Promise((resolve, reject) => {
      fs.writeFile(filePath, content, (err) => {
        if(err) return reject(`Erro ao escrever no arquivo: ${filePath}`);
        return resolve();
      })
    });
  }
  
  static isDevMode() {
    const { NODE_ENV = 'development' } = process.env;
    const executionMode = ['dev', 'prod'].includes(process.argv[2]) ? process.argv[2] : null;
    return executionMode === 'dev' || (!executionMode && NODE_ENV === 'development');
  }

  static replaceMnemonics(text, params = {}) {
    return Object.keys(params).reduce((textToReplace, paramName) => {
      textToReplace = textToReplace.replace(`<>${paramName}<>`, params[paramName]);
      return textToReplace;
    }, text);
  }

  static async build() {
    try {
      const forceBuild = process.argv.includes('force');
      const envFile = path.join(backendPath, '.env');
      
      if (!forceBuild) {
        const fileExists = await EnvBuilder.fileExists(envFile);
        if(fileExists) return;
      }
      
      const envProdFile = path.join(backendPath, '.env.production');
      const envDevFile = path.join(backendPath, '.env.development');
      const envExampleFile = path.join(backendPath, '.env.example');
      
      const envExampleStr = await EnvBuilder.readFile(envExampleFile);
      const fileNameDefaultValues = EnvBuilder.isDevMode() ? envDevFile : envProdFile;
      const envValuesStr = await EnvBuilder.readFile(fileNameDefaultValues);
      
      let defaultValues = dotenv.parse(envValuesStr);
      
      if(EnvBuilder.isDevMode()) {
        defaultValues = { ...defaultValues, ...EnvBuilder.DEFAULT_VALUES };
      }

      const envReplace = EnvBuilder.replaceMnemonics(envExampleStr, defaultValues);
      EnvBuilder.writeFile(envFile, envReplace);
    } catch(error) {
      console.error(error.message);
    }
  }
}

EnvBuilder.build();
