const app = require('./src/main/app');
const settings = require('./src/main/settings');
const cluster = require('cluster');
const os = require('os');

const numCPUs = os.cpus().length;

let server;

if (cluster.isMaster) {
  console.log(`Master process ${process.pid} is running on port ${settings.PORT}`);

  for (let i = 0; i < numCPUs; i++) {
    cluster.fork();
  }

  cluster.on('exit', (worker, code, signal) => {
    console.log(`Worker process ${worker.process.pid} died. Restarting...`);
    cluster.fork();
  });
} else {
  app.set('port', settings.PORT);
  server = app.listen(app.get('port'), () => {});
}

const startGracefulShutdown = () => {
  server.close(() => {
    process.exit(0);
  });
}

process.on('SIGTERM', startGracefulShutdown);
process.on('SIGINT', startGracefulShutdown);
