const Query = require('../service/query');
const Util = require('../service/util');
const Crypt = require('../service/crypt');

const route = (router) => {
  router.post('/sales', async function(req, res, next) {
    try {
      const header = req.headers;

      if (!('authorization' in header)) {
        throw new Error(`Missing authorization in header request`);
      }

      const authorization = header.authorization;

      const token = Util.isValidBearerAuth(authorization);
      const { DB_CRYPT_KEY } = process.env;

      const connParams = JSON.parse(Crypt.decrypt(token, DB_CRYPT_KEY));

      const body = req.body; 
      
      if (!('startDate' in body)) {
        throw new Error(`Missing 'startDate' in body request params.`);
      }

      Util.dateValidation(body.startDate, 'startDate');
      Util.startDateValidation(body.startDate);

      if ('endDate' in body) {
        Util.dateValidation(body.endDate, 'endDate');
        Util.endDateValidation(body.endDate);
      } else {
        const today = new Date();
        const yesterday = new Date(today);
        yesterday.setDate(today.getDate() - 1);

        body.endDate = Util.convertDate(yesterday, 'dd/mm/yyyy');
      }

      Util.startDateBeforeEndDate(body.startDate, body.endDate);
      Util.dateRangeValidation(body.startDate, body.endDate, process.env.MAX_INTERVAL_JSON);

      const params = {
        "P_DTINICIAL": body.startDate,
        "P_DTFINAL": body.endDate
      };

      const sql = Query.get('getSales');
      const data = await Query.run(connParams, sql, params);

      res.json({
        "error": false,
        "data": data.rows
      });
    } catch (error) {
      res.json({
        "error": true,
        "message": error.message
      });
    }
  });
}

module.exports = route;
